@@ -54,6 +54,7 @@ const float PHASE_SCALE   = 0.5f;
 const float OCTAVE_SCALE  = 12.0f;
 const float TUNING_SCALE  = 1.0f;
 const float SWEEP_SCALE   = 0.5f;
+const float PITCH_SCALE   = 0.5f;
 
 const float LFO_FREQ_MIN  = 0.4f;
 const float LFO_FREQ_MAX  = 40.0f;
@@ -79,8 +80,6 @@ inline float synthv1_sigmoid_0 ( const float x, const float t0 )
 {
 	const float t1 = 1.0f - t0;
 #if 0
-	return (x < -1.0f ? -t1 : (x > +1.0f ? t1 : t1 * x * (1.5f - 0.5f * x * x)));
-#else
 	if (x > +t1)
 		return +t1 + t0 * synthv1_tanhf(+(x - t1) / t0);
 	else
@@ -88,6 +87,8 @@ inline float synthv1_sigmoid_0 ( const float x, const float t0 )
 		return -t1 - t0 * synthv1_tanhf(-(x + t1) / t0);
 	else
 		return x;
+#else
+	return (x < -1.0f ? -t1 : (x > +1.0f ? t1 : t1 * x * (1.5f - 0.5f * x * x)));
 #endif
 }
 
@@ -105,6 +106,17 @@ inline float synthv1_velocity ( const float x, const float p = 0.2f )
 }
 
 
+// pitchbend curve
+
+inline float synthv1_pow2f ( const float x )
+{
+// simplest power-of-2 straight linearization
+// -- x argument valid in [-1, 1] interval
+//	return 1.0f + (x < 0.0f ? 0.5f : 1.0f) * x;
+	return ::powf(2.0f, x);
+}
+
+
 // envelope
 
 struct synthv1_env
@@ -247,17 +259,19 @@ struct synthv1_ctl
 	void reset()
 	{
 		pressure = 0.0f;
-		pitchbend = 0.0f;
-		modwheel = 0.0f;
+		pitchbend1 = pitchbend2 = 1.0f;
+		modwheel1 = modwheel2 = 0.0f;
 		panning = 0.0f;
 		volume = 1.0f;
+		sustain = false;
 	}
 
 	float pressure;
-	float pitchbend;
-	float modwheel;
+	float pitchbend1, pitchbend2;
+	float modwheel1, modwheel2;
 	float panning;
 	float volume;
+	bool  sustain;
 };
 
 
@@ -671,6 +685,8 @@ struct synthv1_voice : public synthv1_list<synthv1_voice>
 
 	synthv1_glide dco1_glide1, dco1_glide2;	// glides (portamento)
 	synthv1_glide dco2_glide1, dco2_glide2;
+
+	bool sustain;
 };
 
 
@@ -713,6 +729,7 @@ protected:
 	void allSoundOff();
 	void allControllersOff();
 	void allNotesOff();
+	void allSustainOff();
 
 	synthv1_voice *alloc_voice ()
 	{
@@ -1210,7 +1227,7 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	// note on
 	if (status == 0x90 && value > 0) {
 		synthv1_voice *pv = m_notes[key];
-		if (pv) {
+		if (pv/* && !m_ctl.sustain*/) {
 			// retrigger fast release
 			m_dcf1.env.note_off_fast(&pv->dcf1_env);
 			m_dcf2.env.note_off_fast(&pv->dcf2_env);
@@ -1285,6 +1302,8 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			frames = uint32_t(*m_dco2.glide * *m_dco2.glide * srate);
 			pv->dco2_glide1.reset(frames, pv->dco2_freq1);
 			pv->dco2_glide2.reset(frames, pv->dco2_freq2);
+			// sustain
+			pv->sustain = false;
 			// allocated
 			m_notes[key] = pv;
 		}
@@ -1293,25 +1312,32 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	else if (status == 0x80 || (status == 0x90 && value == 0)) {
 		synthv1_voice *pv = m_notes[key];
 		if (pv && pv->note >= 0) {
-			if (pv->dca1_env.stage != synthv1_env::Release) {
-				m_dca1.env.note_off(&pv->dca1_env);
-				m_dcf1.env.note_off(&pv->dcf1_env);
-				m_lfo1.env.note_off(&pv->lfo1_env);
-			}
-			if (pv->dca2_env.stage != synthv1_env::Release) {
-				m_dca2.env.note_off(&pv->dca2_env);
-				m_dcf2.env.note_off(&pv->dcf2_env);
-				m_lfo2.env.note_off(&pv->lfo2_env);
+			if (m_ctl.sustain) {
+				pv->sustain = true;
+			} else {
+				if (pv->dca1_env.stage != synthv1_env::Release) {
+					m_dca1.env.note_off(&pv->dca1_env);
+					m_dcf1.env.note_off(&pv->dcf1_env);
+					m_lfo1.env.note_off(&pv->lfo1_env);
+				}
+				if (pv->dca2_env.stage != synthv1_env::Release) {
+					m_dca2.env.note_off(&pv->dca2_env);
+					m_dcf2.env.note_off(&pv->dcf2_env);
+					m_lfo2.env.note_off(&pv->lfo2_env);
+				}
 			}
 		}
 	}
 	// control change
 	else if (status == 0xb0) {
 		switch (key) {
-		case 0x01:
+		case 0x01: {
 			// modulation wheel (cc#1)
-			m_ctl.modwheel = float(value) / 127.0f;
+			const float modwheel = float(value) / 127.0f;
+			m_ctl.modwheel1 = *m_def1.modwheel * modwheel;
+			m_ctl.modwheel2 = *m_def2.modwheel * modwheel;
 			break;
+		}
 		case 0x07:
 			// channel volume (cc#7)
 			m_ctl.volume = float(value) / 127.0f;
@@ -1320,6 +1346,12 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			// channel panning (cc#10)
 			m_ctl.panning = float(value - 64) / 64.0f;
 			break;
+		case 0x40:
+			// sustain/damper pedal (cc#64)
+			if (m_ctl.sustain && value <  64)
+				allSustainOff();
+			m_ctl.sustain = bool(value >= 64);
+			break;
 		case 0x78:
 			// all sound off (cc#120)
 			allSoundOff();
@@ -1336,7 +1368,9 @@ void synthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	}
 	// pitch bend
 	else if (status == 0xe0) {
-		m_ctl.pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
+		const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
+		m_ctl.pitchbend1 = synthv1_pow2f(*m_def1.pitchbend * pitchbend);
+		m_ctl.pitchbend2 = synthv1_pow2f(*m_def2.pitchbend * pitchbend);
 	}
 }
 
@@ -1390,6 +1424,30 @@ void synthv1_impl::allNotesOff (void)
 }
 
 
+// all sustained notes off
+
+void synthv1_impl::allSustainOff (void)
+{
+	synthv1_voice *pv = m_play_list.next();
+	while (pv) {
+		if (pv->note >= 0 && pv->sustain) {
+			pv->sustain = false;
+			if (pv->dca1_env.stage != synthv1_env::Release) {
+				m_dca1.env.note_off(&pv->dca1_env);
+				m_dcf1.env.note_off(&pv->dcf1_env);
+				m_lfo1.env.note_off(&pv->lfo1_env);
+			}
+			if (pv->dca2_env.stage != synthv1_env::Release) {
+				m_dca2.env.note_off(&pv->dca2_env);
+				m_dcf2.env.note_off(&pv->dcf2_env);
+				m_lfo2.env.note_off(&pv->lfo2_env);
+			}
+		}
+		pv = pv->next();
+	}
+}
+
+
 // all reset clear
 
 void synthv1_impl::reset (void)
@@ -1447,16 +1505,12 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 	// controls
 
-	const float lfo1_rate  = *m_lfo1.rate * *m_lfo1.rate;
-	const float lfo2_rate  = *m_lfo2.rate * *m_lfo2.rate;
-	const float lfo1_freq  = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
-	const float lfo2_freq  = LFO_FREQ_MIN + lfo2_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
-	const float lfo1_pitch = *m_lfo1.pitch * *m_lfo1.pitch;
-	const float lfo2_pitch = *m_lfo2.pitch * *m_lfo2.pitch;
-	const float modwheel1  = *m_def1.modwheel * (lfo1_pitch + m_ctl.modwheel);
-	const float modwheel2  = *m_def2.modwheel * (lfo2_pitch + m_ctl.modwheel);
-	const float pitchbend1 = (1.0f + *m_def1.pitchbend * m_ctl.pitchbend);
-	const float pitchbend2 = (1.0f + *m_def2.pitchbend * m_ctl.pitchbend);
+	const float lfo1_rate = *m_lfo1.rate * *m_lfo1.rate;
+	const float lfo2_rate = *m_lfo2.rate * *m_lfo2.rate;
+	const float lfo1_freq = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
+	const float lfo2_freq = LFO_FREQ_MIN + lfo2_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
+	const float modwheel1 = m_ctl.modwheel1 + PITCH_SCALE * *m_lfo1.pitch;
+	const float modwheel2 = m_ctl.modwheel2 + PITCH_SCALE * *m_lfo2.pitch;
 
 	if (int(*m_dco1.shape1) != int(dco1_wave1.shape()) || *m_dco1.width1 != dco1_wave1.width())
 		dco1_wave1.reset(synthv1_wave::Shape(*m_dco1.shape1), *m_dco1.width1);
@@ -1544,17 +1598,17 @@ void synthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				float dco22 = pv->dco2_sample2 * pv->dco2_bal.value(j, 1);
 
 				pv->dco1_sample1 = pv->dco1_osc1.sample(pv->dco1_freq1
-					* (pitchbend1 + modwheel1 * lfo1)
+					* (m_ctl.pitchbend1 + modwheel1 * lfo1)
 					+ pv->dco1_glide1.tick());
 				pv->dco1_sample2 = pv->dco1_osc2.sample(pv->dco1_freq2
-					* (pitchbend1 + modwheel1 * lfo1)
+					* (m_ctl.pitchbend1 + modwheel1 * lfo1)
 					+ pv->dco1_glide2.tick());
 
 				pv->dco2_sample1 = pv->dco2_osc1.sample(pv->dco2_freq1
-					* (pitchbend2 + modwheel2 * lfo2)
+					* (m_ctl.pitchbend2 + modwheel2 * lfo2)
 					+ pv->dco2_glide1.tick());
 				pv->dco2_sample2 = pv->dco2_osc2.sample(pv->dco2_freq2
-					* (pitchbend2 + modwheel2 * lfo2)
+					* (m_ctl.pitchbend2 + modwheel2 * lfo2)
 					+ pv->dco2_glide2.tick());
 
 				pv->lfo1_sample = pv->lfo1_osc.sample(lfo1_freq
